import mx.utils.Delegate;

/* Button class
 *  A small streamlined Button component with live-preview and custom component potential.
 * 	This was built to avoid the IDE components and for something powerful/lightweight/scalable
 * 	and custom for my application needs.
 * 
 * Author: Jim Palmer - jimpalmer@gmail.com - www.overset.com 
 * Version 0.3 January 2007
 *   added another beveled edge that can change color when changing draw states
 *   fixed drawing of arrows - for better precision
 * 
 * Version 0.2 December 2006
 * 	added support for _on() and _off() functions which can be called by an observer, MVC singleton
 * 	added compensation for custom stroke width and how it offsets the size of the button as it is an MC
 *   added shadow text for better visual appeal
 * 
 * Version 0.1 July 2006
 *	initial proof of concept and around 99% usable
 * 
 * License: BSD License
 * */

class Btn extends MovieClip {

	private var __width:Number;
	private var __height:Number;

	[Inspectable(name="button text", defaultValue="BUTTON", type=String)]
	private var _bGlyph;
	[Inspectable(name="can get focus?", defaultValue=true, type=Boolean)]
	private var _gainFocus:Boolean = false;
	
	private var _out_hiColor = 0x808080;
	private var _out_loColor = 0x000000;
	private var _out_txtColor = 0x000000;
	private var _out_inBorderColor = 0xFFFFFF;

	private var _on_hiColor = 0x808080;
	private var _on_loColor = 0x808080;
	private var _on_txtColor = 0x808080;
	private var _on_inBorderColor = 0xCCCCCC;

	private var _press_hiColor = 0x808080;
	private var _press_loColor = 0x808080;
	private var _press_txtColor = 0xAAAAAA;
	private var _press_inBorderColor = 0x99FF99;
	

	private var _origGlyph;
	private var _valueMC:MovieClip;
	private var _valueBackgroundMC:MovieClip;
	private var _maskMC:MovieClip;
	
	private var __onRelease;
	private var __onReleaseOutside;
	[Inspectable(name="edge rounding diameter", defaultValue=10, type=Number)]
	private var __rounded:Number = 10;
	[Inspectable(name="border thickness", defaultValue=0, type=Number)]
	private var __borderThickness:Number = 0;
	
	private var _toggle:Boolean = true;
	
	private var BoundingBox:MovieClip;
	
	function Btn () {
		this.__width = this._width;
		this.__height = this._height;

		// initialize the default MC properties
		this.tabEnabled = true;
		
		this._origGlyph = this._bGlyph;
		
		// assign the stock event handlers for the button
		this.onRollOver = this.renderOver;
		this.onRollOut = this.onReleaseOutside = this.renderOut;
		this.onPress = this.renderPress;
		this.onKeyDown = this.captureKeyDown;
		this.onKeyUp = this.captureKeyUp;
		
		// use the hand cursor by default
		this.useHandCursor = true;
		
		// do not allow the yellow tab-hilight on this Btn:MovieClip
		this._focusrect = false;
		// now draw the inital/default off-state of the button
		this.onLoad = Delegate.create(this, initWrapper);
		
		if (typeof _root.observer != "undefined") {
			_root.observer.addSubscriber(this);
		}
		
	}
	
	function update () {
		trace(this + " UPDATE()");
	}
	
	function initWrapper () {
		this.onLoad = null;
		
		if (typeof _root.observer != "undefined") {
			_root.observer.addSubscriber(this);
		}
		
		this.renderOut();
	}

	function init (width:Number, height:Number, glyph, rounded:Number, borderThickness:Number):Void {
		
		this.__width = this._width = width;
		this.__height = this._height = height;
		this._bGlyph = glyph;
		this._origGlyph = glyph;
		if (rounded)
			this.__rounded = rounded;
		else
			this.__rounded = 0;
		if (borderThickness)
			this.__borderThickness = borderThickness;
		
		this.__onRelease = this.onRelease;
		this.__onReleaseOutside = this.onReleaseOutside;
		
		this.renderOut();
		
		// must absolutely delete the boundingbox
		this.BoundingBox.unloadMovie();
		this.BoundingBox.removeMovieClip();
		// issue with the bounding box not being removed on above command
		this.BoundingBox._width = 0;
		this.BoundingBox._height = 0;
	}
	
	function drawButton (width:Number, height:Number, x1:Number, y1:Number, hiColor, loColor, txtColor, inBorderColor) {
		this.clear();
		
		// make sure the scale is 0 and the positioning is integer not rational for text to appear non-fuzzy and clear
		this._yscale = 100;
		this._xscale = 100;
		this._x = Math.round(this._x);
		this._y = Math.round(this._y);

		var margin = (this.__rounded / 2);
		var mos = (this.__borderThickness == 0) ? (0.5) : (this.__borderThickness / 2);
		var whitemos = (mos * 3);

		// now draw the gradient-fill and border
		var topColors = [0x555577, 0xEEEEEF];
		var alphas = [100, 100];
		var ratios = [0, 255];
		// rotate the gradient - (-45degrees) in radians
		var topMatrix = {matrixType:"box", x:x1-(width/2), y:y1+(height/3), w:(width*2), h:height+(height+height), r:180*Math.PI};
		
		// hairline is 1pixel width - adjust for it?
		this.lineStyle(this.__borderThickness, hiColor, 100);
		
		this.beginGradientFill("radial", topColors, alphas, ratios, topMatrix);
		this.moveTo(x1+mos, (y1+margin+mos));
		this.curveTo(x1+mos, y1+mos, (x1+margin+mos), y1+mos)
		this.lineTo((x1+width)-margin-mos, y1+mos);
		this.curveTo((x1+width)-mos, y1+mos, (x1+width)-mos, (y1+margin+mos));
		this.lineTo((x1+width)-mos, ((y1+height)-margin-mos));
		this.lineStyle(this.__borderThickness, hiColor, 0);
		this.lineStyle(this.__borderThickness, hiColor, 100);
		this.curveTo((x1+width)-mos, (y1+height)-mos, (x1+width)-margin-mos, (y1+height)-mos);
		this.lineTo((x1+margin)+mos, (y1+height)-mos);
		this.curveTo(x1+mos, (y1+height)-mos, x1+mos, (y1+height)-margin-mos);
		this.lineTo(x1+mos, (y1+margin+mos))
		this.endFill();

		// draw hilights
		this.lineStyle(0, inBorderColor, 50);
		this.moveTo(x1+whitemos, (y1+margin+mos));
		this.lineTo(x1+whitemos, (y1+height)-margin-mos);
		this.curveTo(x1+whitemos, (y1+height)-whitemos, x1+margin+mos, (y1+height)-whitemos);
		this.lineTo((x1+width)-margin-whitemos, (y1+height)-whitemos);
		this.curveTo((x1+width)-whitemos, (y1+height)-whitemos, (x1+width)-whitemos, (y1+height)-margin-mos);
		this.lineTo((x1+width)-whitemos, y1+margin+mos);
		this.curveTo((x1+width)-whitemos, y1+whitemos, (x1+width)-margin-whitemos, y1+whitemos);
		this.lineTo(x1+margin+whitemos, y1+whitemos);
		this.curveTo(x1+whitemos, y1+whitemos, x1+whitemos, (y1+margin+mos));

		whitemos += mos + 0.15;
		this.lineStyle(0, inBorderColor, 15);
		this.moveTo(x1+whitemos, (y1+margin+mos));
		this.lineTo(x1+whitemos, (y1+height)-margin-mos);
		this.curveTo(x1+whitemos, (y1+height)-whitemos, x1+margin+mos, (y1+height)-whitemos);
		this.lineTo((x1+width)-margin-whitemos, (y1+height)-whitemos);
		this.curveTo((x1+width)-whitemos, (y1+height)-whitemos, (x1+width)-whitemos, (y1+height)-margin-mos);
		this.lineTo((x1+width)-whitemos, y1+margin+mos);
		this.curveTo((x1+width)-whitemos, y1+whitemos, (x1+width)-margin-whitemos, y1+whitemos);
		this.lineTo(x1+margin+whitemos, y1+whitemos);
		this.curveTo(x1+whitemos, y1+whitemos, x1+whitemos, (y1+margin+mos));

		
		if (typeof(this[this._bGlyph]) == "function") {
			this[this._bGlyph](txtColor);
		} else if ( (typeof(this._bGlyph) == "string" && this._bGlyph.length > 0) || typeof(this._bGlyph) == "object") {
			// create empty clip for resizing text issues
			this._valueMC.removeMovieClip();
			this._valueMC = createEmptyMovieClip("_valueMC", this.getNextHighestDepth());
			var BSName:String = "btnShadowTxt";
			var BTName:String = "btnTxt";
			this._valueMC.createTextField(BSName, this._valueMC.getNextHighestDepth(), 0, 0, width, height);
			this._valueMC.createTextField(BTName, this._valueMC.getNextHighestDepth(), 0, 0, width, height);
			var BS = this._valueMC[BSName];
			var BT = this._valueMC[BTName];
			
			if (typeof(this._bGlyph) == "string") {
				BT.text = BS.text = this._bGlyph;
			} else if (typeof(this._bGlyph.text) == "string") {
				BT.text = BS.text = this._bGlyph.text;
			}
			
			BT.border = BS.border = false;
			BT.type = BS.type = "dynamic";
			BT.html = BS.html = false;
			BT.embedFonts = BS.embedFonts = true;
			BT.multiline = BS.multiline = false;
			BT.mouseWheelEnabled = BS.mouseWheelEnabled = false;
			BT.background = BS.background = false;
			BT.selectable = BS.selectable = false;
			BT.wordWrap = BS.wordWrap = false;
			
			var btnTxtFormat:TextFormat = new TextFormat();
			btnTxtFormat.font = "uni 05_53";
			btnTxtFormat.size = 8;
			btnTxtFormat.color = 0xFFFFFF;
			BS.setTextFormat(btnTxtFormat);
			BS.setNewTextFormat(btnTxtFormat);
			BS.autoSize = true;
			btnTxtFormat.color = txtColor;
			BT.setTextFormat(btnTxtFormat);
			BT.setNewTextFormat(btnTxtFormat);
			BT.autoSize = true;
			
			// vertically align the text
			BT._y = Math.round( ((height / 2) - (btnTxtFormat.size)) );
			// obviously flash pads their textfield and doesn't resize in relation to a late-assignment of the textFormat .size attribute
			BT._height = btnTxtFormat.size;
			BT._x = Math.round( ((width / 2) - (BT.textWidth / 2) - 1) );
			//BT._width = BT.textWidth;
			BT._rotation = this._bGlyph.rotation;
			
			BS._alpha = 50;
			BS._x = BT._x + 1;
			BS._y = BT._y + 1;
		}		

	}
	
	function renderOut () {
		if (this._toggle == false) return false;
		this.drawButton(this.__width, this.__height, 0, 0, this._out_hiColor, this._out_loColor, this._out_txtColor, this._out_inBorderColor);
	}

	function renderOver () {
		if (this._toggle == false) return false;
		this.drawButton(this.__width, this.__height, 0, 0, this._on_hiColor, this._on_loColor, this._on_txtColor, this._on_inBorderColor);
	}

	function renderPress () {
		if (this._toggle == false) return false;
		this.drawButton(this.__width, this.__height, 0, 0, this._press_hiColor, this._press_loColor, this._press_txtColor, this._press_inBorderColor);
		// set focus on this button ONLY if it has a text glyph
		if (this._gainFocus) {
			Selection.setFocus(this);
		}
	}
	
	function captureKeyUp () {
		if (this._toggle == false) return false;
		// capture enter=13 and space=32
		if (Key.getCode() == 32 || Key.getCode() == 13) {
			this.onRelease();
		} else if (Key.getCode() == 9) {
			// else they hit tab to highlight the button via tabOrder
			this.renderOver();
		}
	}

	function captureKeyDown () {
		if (this._toggle == false) return false;
		if (Key.getCode() == 32 || Key.getCode() == 13) {
			this.renderPress();
		}
	}
	
	function _off (offGlyph) {
		this._toggle = false;
		this.__onRelease = this.onRelease;
		this.__onReleaseOutside = this.onReleaseOutside;
		this.onRelease = null;
		this.onReleaseOutside = null;
		/* XXX DEBUG 
			trace(this + " OFF -- " + typeof this.onRelease);
		*/
		this.drawButton(this.__width, this.__height, 0, 0, this._out_hiColor, this._out_loColor, this._out_txtColor, this._out_inBorderColor);
	}

	function _on () {
		this._toggle = true;
		if (typeof this.__onRelease != "undefined") this.onRelease = this.__onRelease;
		if (typeof this.__onReleaseOutside != "undefined") this.onReleaseOutside = this.__onReleaseOutside;
		/* XXX DEBUG 
			trace(this + " ON -- " + typeof this.onRelease);
		*/
		this.drawButton(this.__width, this.__height, 0, 0, this._out_hiColor, this._out_loColor, this._out_txtColor, this._out_inBorderColor);
	}
	
	/* simple glyph drawing functions */
	
	// down arrow
	function _drawDownArrow (Color) {
		var size:Number = Math.round((this.__width / 2) - 2);
		this.lineStyle(0, Color, 100);
		var startY = Math.round((this.__height / 2) - (size / 2));
		var startX = Math.round((this.__width / 2) - (size / 2));
		this.moveTo(startX + 0.15, startY + 0.15);
		this.beginFill(Color);
		this.lineTo(Math.round(startX + (size / 2)), (size + startY - 1));
		this.lineTo((size + startX), startY);
		this.lineTo(startX, startY);
		this.endFill();
	}
	// open down arrow
	function _drawOpenDownArrow (Color) {
		var size:Number = Math.round((this.__width / 2) - 2);
		this.lineStyle(0, Color, 100);
		var startY = Math.round((this.__height / 2) - (size / 2));
		var startX = Math.round((this.__width / 2) - (size / 2));
		this.moveTo(startX + 0.15, startY + 0.15);
		//this.beginFill(Color);
		this.lineTo(Math.round(startX + (size / 2)), (size + startY - 1));
		this.lineTo((size + startX), startY);
		//this.lineTo(startX, startY);
		//this.endFill();
	}
	
	// up arrow
	function _drawUpArrow (Color) {
		var size:Number = Math.round((this.__width / 2) - 2);
		this.lineStyle(0, Color, 100);
		var startY = Math.round((this.__height / 2) + (size / 2));
		var startX = Math.round((this.__width / 2) - (size / 2));
		this.moveTo(startX + 0.15, startY + 0.15);
		this.beginFill(Color);
		this.lineTo(Math.round(startX + (size / 2)), (startY - size + 1));
		this.lineTo((size + startX), startY);
		this.lineTo(startX, startY);
		this.endFill();
	}
	
}